﻿// created by Josh Heyde
// last updated 27/01/09

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;

namespace Memories
{
    public partial class frmCreateMemory : Form
    {
        // a global list to store the images
        List<string> l_images = new List<string>();

        // a global position relating to the current image being displayed
        int count = 0;

        // a global variable to check if the memory has already been saved
        bool saved = false;

        public frmCreateMemory()
        {
            InitializeComponent();
        }

        // form closing event handler
        private void Form2_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!saved)
            {
                // warns the user they will lose their data if they do not save
                DialogResult dr = MessageBox.Show("Do you wish to save this memory before closing?\nAll data will be lost if not",
                                                    "Memory Close", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);

                // performs the appropriate action depending on the button clicked on the dialog
                if (dr == DialogResult.Cancel)
                    e.Cancel = true;
                else
                    if (dr == DialogResult.Yes)
                    {
                        // ensures validation conditions have been met
                        if (!ValidateChildren(ValidationConstraints.Visible))
                            DialogResult = DialogResult.None;
                        else
                        {
                            // prevents the form from closing if the save dialog is closed
                            if (!Save())
                                e.Cancel = true;
                        }
                    }
            }
        }

        // description textbox validation
        private void tbxDescription_Validating(object sender, CancelEventArgs e)
        {
            string errMag = null;

            // ensures that the description of the memory contains a value before saving
            if (tbxDescription.Text.Trim().Length == 0)
            {
                errMag = "Please enter a description.";
                e.Cancel = true;
            }

            errorProvider1.SetError(tbxDescription, errMag);
        }

        // validation of the picture box for the current image
        private void pbxMemImage1_Validating(object sender, CancelEventArgs e)
        {
            string errMag = null;

            // ensures that an image is selected before saving
            if (pbxMemImage1.Image == null)
            {
                errMag = "Please select a picture.";
                e.Cancel = true;
            }

            errorProvider1.SetError(btnSelectImages, errMag);
        }

        // event handler for clicking the select image button
        private void btnSelectImages_Click(object sender, EventArgs e)
        {
            openFileDialog1.FileName = "";

            // sets the image in the picture box
            DialogResult dr = openFileDialog1.ShowDialog();
            try
            {
                if (dr == DialogResult.OK)
                {
                    // allows multiple files to inserted
                    for (int i = 0; i < openFileDialog1.FileNames.Length; i++)
                        l_images.Add(openFileDialog1.FileNames[i]);

                    // ensures capacity is actually equal to the size
                    l_images.TrimExcess();

                    // displays the first image
                    pbxMemImage1.ImageLocation = l_images[count];

                    // enables the buttons to cycle through the images if more than image is
                    // added to the memory
                    if (l_images.Count > 1)
                    {
                        btnPrevious.Enabled = true;
                        btnNext.Enabled = true;
                    }
                    else
                    {
                        btnPrevious.Enabled = false;
                        btnNext.Enabled = false;
                    }

                    // informs the user how many images they have and which one they
                    // are currently viewing
                    lblImageInfo.Text = "Image " + (count + 1) + " of " + l_images.Count;
                }
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Error", MessageBoxButtons.OKCancel, MessageBoxIcon.Error);
            }
        }

        // event handler for clicking the save button
        private void btnSave_Click(object sender, EventArgs e)
        {
            // ensures validation conditions have been met
            if (!ValidateChildren(ValidationConstraints.Visible))
                return;

            // asks the user if they are sure if they want to save
            DialogResult dr = MessageBox.Show("Are you sure you wish to save?\nYou will be unable to edit this memory afterwards.",
                                                "Save Memory", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation);
            try
            {
                // performs the appropriate action depending on the button clicked on the dialog
                if (dr != DialogResult.Yes)
                    return;
                else
                {
                    // prevents the form from closing if the save dialog is closed
                    if (Save())
                        this.Close();
                }
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Error", MessageBoxButtons.OKCancel, MessageBoxIcon.Error);
            }
        }

        // event handler for clicking the close button
        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        // a public function to save the memory
        public bool Save()
        {
            // sets up a default name for the save dialog
            saveFileDialog1.FileName = this.Text;

            // stores the result of the file dialog
            DialogResult dr = saveFileDialog1.ShowDialog();

            // performs the appropriate action depending on the button clicked on the dialog
            if (dr != DialogResult.OK)
                return false;
            else
            {
                // attempts to save the relevant data to a .mem file
                try
                {
                    StreamWriter sw = new StreamWriter(saveFileDialog1.FileName);

                    // prevents any commas being saved into the description
                    // by removing them
                    string[] tempDescriptionArr = Regex.Split(tbxDescription.Text, @",");
                    string tempDescription = "";

                    for(int i = 0; i < tempDescriptionArr.Length; i++)
                        tempDescription += (tempDescriptionArr[i] + " ");

                    sw.Write(tempDescription);
                    sw.Write(",");
                    sw.Write(dtpMemDate.Value);

                    // writes all images to the file
                    for (int i = 0; i < l_images.Capacity; i++)
                    {
                        sw.Write(",");
                        sw.Write(l_images[i]);
                    }
                    
                    sw.Close();

                    saved = true;

                    return true;
                }
                catch(Exception exc)
                {
                    MessageBox.Show(exc.Message, "Unable to write to file", MessageBoxButtons.OKCancel, MessageBoxIcon.Error);
                    return false;
                }
            }
         }

        // event handler for clicking the previous button
        private void btnPrevious_Click(object sender, EventArgs e)
        {
            // cycles through images backwards
            if (--count < 0)
                count = l_images.Count - 1;

            pbxMemImage1.ImageLocation = l_images[count];

            // informs the user how many images they have and which one they
            // are currently viewing
            lblImageInfo.Text = "Image " + (count + 1) + " of " + l_images.Count;
        }

        // event handler for clicking the next button
        private void btnNext_Click(object sender, EventArgs e)
        {
            // cycles through the images forwards
            if (++count > l_images.Count - 1)
                count = 0;

            pbxMemImage1.ImageLocation = l_images[count];

            // informs the user how many images they have and which one they
            // are currently viewing
            lblImageInfo.Text = "Image " + (count + 1) + " of " + l_images.Count;
        }

        // event handler for clicking the clear images button
        private void btnClearImages_Click(object sender, EventArgs e)
        {
            // empties the list of image locations to allow the user to start anew
            l_images.Clear();
            pbxMemImage1.ImageLocation = null;
            btnPrevious.Enabled = false;
            btnNext.Enabled = false;
            count = 0;
            l_images.TrimExcess();

            // informs the user how many images they have and which one they
            // are currently viewing
            lblImageInfo.Text = "Image " + count + " of " + l_images.Count;
        }

        // form load event handler
        private void Form2_Load(object sender, EventArgs e)
        {
            // prevents the user setting a date beyond the current date
            dtpMemDate.MaxDate = DateTime.Today;
        }

        // form activated event handler
        private void Form2_Activated(object sender, EventArgs e)
        {
            // sets the focus to the description textbox
            tbxDescription.Focus();
        }
    }
}
