﻿// created by Josh Heyde
// last updated 27/01/09

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;

namespace Memories
{
    public partial class frmDisplayMemory : Form
    {
        // a global list to store the image locations
        List<string> l_images = new List<string>();

        // three global countera to keep track of the images being displayed
        int count1 = 0;
        int count2 = 1;
        int count3 = 2;

        // global booleans to determine whether how many picture boxes will need to display an image
        bool twoImages = false;
        bool threePlusImages = false;

        public frmDisplayMemory()
        {
            InitializeComponent();
        }

        // a function to set all of the data correctly in the 
        public void readData(string fileName)
        {
            try
            {
                // creating access to the file
                StreamReader sr = new StreamReader(fileName);

                // storing all the data in a string
                string temp = sr.ReadLine();

                // closing access to the file
                sr.Close();

                string[] tempData = Regex.Split(temp, @",");

                for (int i = 0; i < tempData.Length; i++)
                {
                    if (tempData[i].Trim().Length == 0)
                        throw new Exception("Empty field detected, failed to open file.");
                }

                tbxDescription.Text = tempData[0];
                mtbDate.Text = tempData[1];
                pbxMemory1.ImageLocation = tempData[2];

                // ensures that only the correct amount of images are displayed
                if (tempData.Length > 3)
                {
                    pbxMemory2.ImageLocation = tempData[3];
                    twoImages = true;
                    if (tempData.Length > 4)
                    {
                        pbxMemory3.ImageLocation = tempData[4];
                        threePlusImages = true;
                    }
                }

                // writing the image locations to the global list
                for(int i = 2; i < tempData.Length; i++)
                    l_images.Add(tempData[i]);

                // ensuring the capacity is the same as the number of images
                l_images.TrimExcess();

                // enables the buttons to cycle through the images
                // if the capacity is large enough
                if (l_images.Count > 1)
                {
                    btnPrevious.Enabled = true;
                    btnNext.Enabled = true;
                }
            }
            catch(Exception exc)
            {
                MessageBox.Show(exc.Message + "\nMemory will now close", "\nUnable to display memory correctly", MessageBoxButtons.OKCancel, MessageBoxIcon.Error);
                this.Close();
            }
        }

        // closes the form
        private void closeMewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        // cycles to the previous image
        private void btnPrevious_Click(object sender, EventArgs e)
        {
            // cycles through images backwards
            if (--count1 < 0)
                count1 = l_images.Count - 1;

            if (--count2 < 0 && (twoImages || threePlusImages))
                count2 = l_images.Count - 1;

            if (--count3 < 0 && threePlusImages)
                count3 = l_images.Count - 1;

            // displays the relevant amount of images for the memory
            pbxMemory1.ImageLocation = l_images[count1];

            if(twoImages || threePlusImages)
                pbxMemory2.ImageLocation = l_images[count2];
            
            if (threePlusImages)
                pbxMemory3.ImageLocation = l_images[count3];
        }

        // cycles to the next image
        private void btnNext_Click(object sender, EventArgs e)
        {
            // cycles through the images forwards
            if (++count1 >= l_images.Count)
                count1 = 0;

            if (++count2 >= l_images.Count)
                count2 = 0;

            if (++count3 >= l_images.Count)
                count3 = 0;

            // displays the relevant amount of images for the memory
            pbxMemory1.ImageLocation = l_images[count1];

            if (twoImages || threePlusImages)
                pbxMemory2.ImageLocation = l_images[count2];

            if (threePlusImages)
                pbxMemory3.ImageLocation = l_images[count3];
        }

        private void frmDisplayMemory_FormClosing(object sender, FormClosingEventArgs e)
        {
            // calls the appropriate method to set the window menu on the main form
            frmMemories f1 = (frmMemories)this.MdiParent;
            f1.setWindowMenu();
        }
    }
}
